'use strict';

var entities = require('html-entities');

var _require = require('immutable'),
    Map = _require.Map;

var _require2 = require('../../'),
    Serializer = _require2.Serializer,
    MARKS = _require2.MARKS;

var _require3 = require('../../utils/escape'),
    escapeWith = _require3.escapeWith;

var xmlEntities = new entities.XmlEntities();

// Replacements for Asciidoc escaping
var REPLACEMENTS = Map([['*', '\\*'], ['#', '\\#'], ['(', '\\('], [')', '\\)'], ['[', '\\['], [']', '\\]'], ['`', '\\`'], ['<', '&lt;'], ['>', '&gt;'], ['_', '\\_'], ['|', '\\|'], ['{', '\\{'], ['}', '\\}']]);

/**
 * Escape a string to asciidoc.
 * @param {String} text
 * @return {String}
 */
function escape(text) {
    text = escapeWith(REPLACEMENTS, text);
    return xmlEntities.encode(text);
}

/**
 * Escape all text leaves during serialization.
 * This step should be done before processing text leaves for marks.
 *
 * @type {Serializer}
 */
var serialize = Serializer().transformText(function (state, leaf) {
    var text = leaf.text,
        marks = leaf.marks;

    var hasCode = marks.some(function (mark) {
        return mark.type === MARKS.CODE;
    });

    return leaf.merge({
        text: hasCode ? text : escape(text, false)
    });
});

module.exports = { serialize: serialize };