'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _unendingTags = require('../../test/unendingTags');

var _unendingTags2 = _interopRequireDefault(_unendingTags);

var _ = require('../');

var _markdown = require('../markdown');

var _markdown2 = _interopRequireDefault(_markdown);

var _html = require('../html');

var _html2 = _interopRequireDefault(_html);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-console */
var PARSERS = {
    '.md': _markdown2.default,
    '.markdown': _markdown2.default,
    '.mdown': _markdown2.default,
    '.html': _html2.default
};

/**
 * Fail with an error message
 * @param  {String} msg
 */
function fail(msg) {
    console.log('error:', msg);
    process.exit(1);
}

/**
 * Execute a transformation over file
 * @param  {Function} fn [description]
 * @return {[type]}      [description]
 */
function transform(fn) {
    if (process.argv.length < 3) {
        fail('no input file');
    }

    var filePath = _path2.default.join(process.cwd(), process.argv[2]);

    var ext = _path2.default.extname(filePath);
    var parser = PARSERS[ext];

    if (!parser) {
        fail('no parser for this file type');
    }

    var content = _fs2.default.readFileSync(filePath, { encoding: 'utf8' });
    var state = _.State.create(parser, { unendingTags: _unendingTags2.default });

    var document = state.deserializeToDocument(content);

    fn(document, state);
}

exports.default = {
    transform: transform
};