'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _splitLines = require('split-lines');

var _splitLines2 = _interopRequireDefault(_splitLines);

var _ = require('../../');

var _block = require('../re/block');

var _block2 = _interopRequireDefault(_block);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Serialize a blockquote node to markdown
 * @type {Serializer}
 */
var serialize = (0, _.Serializer)().matchType(_.BLOCKS.BLOCKQUOTE).then(function (state) {
    var node = state.peek();
    var inner = state.use('block').serialize(node.nodes);
    var lines = (0, _splitLines2.default)(inner.trim());

    var output = lines.map(function (line) {
        return line ? '> ' + line : '>';
    }).join('\n');

    return state.shift().write(output + '\n\n');
});

/**
 * Deserialize a blockquote to a node.
 * @type {Deserializer}
 */
var deserialize = (0, _.Deserializer)().matchRegExp(_block2.default.blockquote, function (state, match) {
    var inner = match[0].replace(/^ *> ?/gm, '').trim();
    var nodes = state.use('block')
    // Signal to children that we are in a blockquote
    .setProp('blockquote', state.depth).deserialize(inner);
    var node = _.Block.create({
        type: _.BLOCKS.BLOCKQUOTE,
        nodes: nodes
    });

    return state.push(node);
});

exports.default = { serialize: serialize, deserialize: deserialize };