'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _ = require('../../');

var _block = require('../re/block');

var _block2 = _interopRequireDefault(_block);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Serialize a math node to markdown
 * @type {Serializer}
 */
var serialize = (0, _.Serializer)().matchType(_.BLOCKS.MATH).then(function (state) {
    var node = state.peek();
    var data = node.data;

    var formula = data.get('formula');

    var output = '$$\n' + formula.trim() + '\n$$\n\n';

    return state.shift().write(output);
});

/**
 * Deserialize a math block into a paragraph with an inline math in it.
 * @type {Deserializer}
 */
var deserialize = (0, _.Deserializer)().matchRegExp(_block2.default.math, function (state, match) {
    var formula = match[2].trim();

    if (state.getProp('math') === false || !formula) {
        return undefined;
    }

    var node = _.Block.create({
        type: _.BLOCKS.MATH,
        isVoid: true,
        data: {
            formula: formula
        }
    });

    return state.push(node);
});

exports.default = { serialize: serialize, deserialize: deserialize };