'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _jsYaml = require('js-yaml');

var _jsYaml2 = _interopRequireDefault(_jsYaml);

var _frontMatter = require('front-matter');

var _frontMatter2 = _interopRequireDefault(_frontMatter);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _ = require('../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Serialize a document to markdown.
 * @type {Serializer}
 */
var serialize = (0, _.Serializer)().matchObject('document').then(function (state) {
    var node = state.peek();
    var data = node.data,
        nodes = node.nodes;

    var body = state.use('block').serialize(nodes);

    if (data.size === 0) {
        return state.shift().write(body);
    }

    var frontMatter = '---\n' + _jsYaml2.default.safeDump(data.toJS(), {
        skipInvalid: true
    }) + '---\n\n';

    return state.shift().write(frontMatter + body);
});

/**
 * Deserialize a document.
 * @type {Deserializer}
 */
var deserialize = (0, _.Deserializer)().then(function (state) {
    var text = state.text;

    var parsed = (0, _frontMatter2.default)(text);

    var nodes = state.use('block').deserialize(parsed.body);
    var data = _immutable2.default.fromJS(parsed.attributes);

    var node = _.Document.create({
        data: data,
        nodes: nodes
    });

    return state.skip(text.length).push(node);
});

exports.default = { serialize: serialize, deserialize: deserialize };