'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _ = require('../../');

var _inline = require('../re/inline');

var _inline2 = _interopRequireDefault(_inline);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Replace hardline break by two spaces followed by a newline
 *
 * @type {Serializer}
 */
var serialize = (0, _.Serializer)().transformText(function (state, leaf) {
    var text = leaf.text;

    var allowHardlineBreak = state.getProp('hardlineBreak');
    var replaceWith = allowHardlineBreak ? '  \n' : ' ';

    return leaf.merge({
        text: text.replace(/\n/g, replaceWith)
    });
});

/**
 * Deserialize hardline break.
 * http://spec.commonmark.org/0.26/#hard-line-break
 *
 * @type {Deserializer}
 */
var deserialize = (0, _.Deserializer)().matchRegExp(_inline2.default.br, function (state, match) {
    return state.pushText('\n');
});

exports.default = { serialize: serialize, deserialize: deserialize };