'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.unescapeWith = exports.escapeWith = undefined;

var _escapeStringRegexp = require('escape-string-regexp');

var _escapeStringRegexp2 = _interopRequireDefault(_escapeStringRegexp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Build a regexp from a string
function re(str) {
    return new RegExp((0, _escapeStringRegexp2.default)(str), 'g');
}

/**
 * Escape a string using a map of replacements.
 * @param  {Map} replacements
 * @param  {String} text
 * @return {String}
 */
function escapeWith(replacements, text) {
    return replacements.reduce(function (out, value, key) {
        return out.replace(re(key), value);
    }, text);
}

/**
 * Unescape a string using a map of replacements.
 * @param  {Map} replacements
 * @param  {String} text
 * @return {String}
 */
function unescapeWith(replacements, text) {
    return escapeWith(replacements.flip(), text);
}

exports.escapeWith = escapeWith;
exports.unescapeWith = unescapeWith;